/* GtkEditor - a source editor widget for GTK
 * Copyright (C) 1998, 2000 Thomas Mailund.
 *
 * The editor widget was written by Thomas Mailund, so bugs should be
 * reported to <mailund@daimi.au.dk>, not the gtk ppl.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GTK_EDITOR_H__
#define __GTK_EDITOR_H__

#include <gdk/gdk.h>
#include <gtk/gtk.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GTK_TYPE_EDITOR (gtk_editor_get_type ())
#define GTK_EDITOR(obj) (GTK_CHECK_CAST (obj, gtk_editor_get_type (), GtkEditor))
#define GTK_EDITOR_CLASS(klass) \
  (GTK_CHECK_CLASS_CAST (klass, gtk_editor_get_type (), GtkEditorClass))
#define GTK_IS_EDITOR(obj) (GTK_CHECK_TYPE (obj, gtk_editor_get_type ()))
#define GTK_IS_EDITOR_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_EDITOR))


  typedef struct _GtkEditor        GtkEditor;
  typedef struct _GtkEditorClass   GtkEditorClass;

  /* one_line_ind is used for one line indenter, and is called when
   * TAB is pressed.  If the function pointer is NULL, this is
   * ignored.  This functionallity is not really implemented yet...the
   * function is called, but there should be some other API.
   */
  typedef void (*GtkEditorIndFunc)(GtkEditor *,gpointer);

  typedef struct _TextProperty {
    GdkFont *font;
    GdkColor *fore;
    GdkColor *back;
  }GtkEditorTextProperty;

  typedef struct _GtkEditorTokenEntry {
    char *name;
    char *pattern;
    int precedence;
    /* id of the block it begins, -1 if it doesn't begin a block*/
    int begin;
    /* ids of the blocks it ends */
    int n_ends;
    int *ends;

    /* the text properties associated with the entry*/
    int prop_id;

  } GtkEditorTokenEntry;

  typedef struct _GtkEditorBlockEntry {
    char *name;
    /* id(s) of the begin / end tokens */
    int begin;
    int n_ends;
    int *ends;

    int precedence;
    int nest;

    /* the text properties associated with the entry*/
    int prop_id;
  } GtkEditorBlockEntry;
  
  typedef struct {
    void (*token_func)(void (*func)(guint16, char *text, guint16));
    void (*char_func)(int (*func)());
    int (*lex)();
  } GtkEditorScanner;
       
  /* --<the widget>------------------------------------------------------- */
  struct _GtkEditor {
    GtkText text;

    /* --<indenter stuff>------------------------------------------------- */
    GtkEditorIndFunc one_line_ind;
    guint ind_accel_key;
    GdkModifierType ind_accel_mod;
    gpointer indenter_data;

    /* --<hilite stuff>--------------------------------------------------- */
    int n_text_properties;
    GtkEditorTextProperty *text_properties;
    int n_token_entries;
    GtkEditorTokenEntry *token_entries;
    int n_block_entries;
    GtkEditorBlockEntry *block_entries;
    /* scanner */
    GtkEditorScanner *scanner;

    /* tokens */
    GList *tokens;
    GList *cur_token;
    int cur_token_pos;

    /* blocks */
    GNode *blocks;
    GNode *cur_block;
    GNode *prev_block;
    GNode *next_block;
  };

  struct _GtkEditorClass {
    GtkTextClass parent_class;
  };


  /* --<prototypes>------------------------------------------------------- */
  /* widget stuff */
  guint          gtk_editor_get_type               (void);
  GtkWidget*     gtk_editor_new                    (GtkAdjustment *hadj,
						    GtkAdjustment *vadj);

  /* syntax highlighting */
  GtkEditorScanner
                *gtk_editor_compile_scanner        (const char    *name,
					            GList         *entries);
  GtkEditorScanner
                *gtk_editor_load_scanner           (const char    *name);
  void           gtk_editor_install_scanner        (GtkEditor     *editor,
					            GtkEditorScanner *scanner,
					            GList            *entries);
  void           gtk_editor_install_bentries       (GtkEditor     *editor,
                                                    GList         *bentries);
  void           gtk_editor_install_text_properties(GtkEditor     *editor,
                                                    GList         *props);
  GList         *gtk_editor_text_property_new      (GdkFont       *font,
						    GdkColor      *fore,
						    GdkColor      *back,
						    GList         *next);
  GList         *gtk_editor_pentry_new             (const gchar   *name,
						    const gchar   *pattern,
						    int            precedence,
						    int            prop_id,
						    GList          *next);

  GList         *gtk_editor_bentry_new             (const gchar    *name,
                                                    int             begin,
						    int             n_ends,
						    int            *end,
						    int             precedence,
						    gboolean        nest,
						    int             prop_id,
						    GList          *next);
  void           gtk_editor_hilite_buffer          (GtkEditor      *editor);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_EDITOR_H__ */
